@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FileCopy.cc,v 1.102 2004/03/19 14:20:55 lav Exp $ */
+/* $Id: FileCopy.cc,v 1.104 2004/09/16 19:17:03 lav Exp $ */
 
 /* FileCopyPeer behaviour:
     1) when suspended, does nothing
@@ -1314,12 +1314,11 @@ int FileCopyPeerFDStream::Do()
       {
 	 if(eof)
 	 {
+	    getfd(); // give it a chance to create empty file. (FIXME - handle tmp errors)
 	    if(!date_set && date!=NO_DATE && do_set_date)
 	    {
 	       if(date==NO_DATE_YET)
 		  return m;
-	       if(getfd()==-1)
-		  return m;
 	       stream->setmtime(date);
 	       date_set=true;
 	       m=MOVED;
