@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FindJob.h,v 1.16 2002/07/31 11:28:00 lav Exp $ */
+/* $Id: FindJob.h,v 1.17 2003/12/01 07:46:19 lav Exp $ */
 
 #ifndef FINDJOB_H
 #define FINDJOB_H
@@ -119,7 +119,7 @@ public:
 
 class FinderJob_List : public FinderJob
 {
-   Buffer *buf;
+   IOBuffer *buf;
    ArgV *args;
 protected:
    prf_res ProcessFile(const char *d,const FileInfo *fi);
