@@ -645,12 +645,14 @@ int Fish::HandleReplies()
    {
       if(state==CONNECTING_1)
       {
-	 const char *p="password: ";
-	 const char *y="(yes/no)? ";
+	 const char *p="password:";
+	 const char *y="(yes/no)?";
 	 int p_len=strlen(p);
 	 int y_len=strlen(y);
+	 if(s>0 && b[s-1]==' ')
+	    s--;
 	 if(s>=p_len && !strncasecmp(b+s-p_len,p,p_len)
-	 || s>10 && !strncmp(b+s-3,"': ",3))
+	 || s>10 && !strncmp(b+s-2,"':",2))
 	 {
 	    if(!pass)
 	    {
@@ -1013,7 +1015,12 @@ int Fish::Write(const void *buf,int size)
    if(size+send_buf->Size()>0x4000)
       size=0x4000-send_buf->Size();
    if(pos+size>entity_size)
+   {
       size=entity_size-pos;
+      // tried to write more than originally requested. Make it retry with Open:
+      if(size==0)
+	 return STORE_FAILED;
+   }
    if(size<=0)
       return 0;
    send_buf->Put((char*)buf,size);
