@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: Http.cc,v 1.160.2.4 2004/01/23 19:17:01 lav Exp $ */
+/* $Id: Http.cc,v 1.173 2004/08/23 14:39:36 lav Exp $ */
 
 #include <config.h>
 #include "trio.h"
@@ -42,7 +42,7 @@
 
 #include "ascii_ctype.h"
 
-#ifndef HAVE_STRPTIME_DECL
+#if !HAVE_DECL_STRPTIME
 CDECL char *strptime(const char *buf, const char *format, struct tm *tm);
 #endif
 
@@ -83,6 +83,7 @@ void Http::Init()
    proto_version=0x10;
    sent_eot=false;
    last_method=0;
+   entity_content_type=0;
 
    default_cwd="/";
 
@@ -197,6 +198,8 @@ void Http::ResetRequestData()
    chunk_pos=0;
    chunked_trailer=false;
    seen_ranges_bytes=false;
+   xfree(entity_content_type);
+   entity_content_type=0;
 }
 
 void Http::Close()
@@ -337,7 +340,7 @@ void Http::SendMethod(const char *method,const char *efile)
       if(!xstrcmp(referer,"."))
       {
 	 referer=GetConnectURL(NO_USER+NO_PASSWORD);
-	 if(referer[0] && referer[strlen(referer)-1]!='/')
+	 if(last_char(referer)!='/')
 	    slash="/";
       }
       if(referer && referer[0])
@@ -350,15 +353,21 @@ void Http::SendMethod(const char *method,const char *efile)
    }
 }
 
-
+void Http::SendBasicAuth(const char *tag,const char *auth)
+{
+   if(!auth || !*auth)
+      return;
+   int auth_len=strlen(auth);
+   char *buf64=string_alloca(base64_length(auth_len)+1);
+   base64_encode(auth,buf64,auth_len);
+   Send("%s: Basic %s\r\n",tag,buf64);
+}
 void Http::SendBasicAuth(const char *tag,const char *user,const char *pass)
 {
    /* Basic scheme */
    char *buf=string_alloca(strlen(user)+1+strlen(pass)+1);
    sprintf(buf,"%s:%s",user,pass);
-   char *buf64=string_alloca(base64_length(strlen(buf))+1);
-   base64_encode(buf,buf64,strlen(buf));
-   Send("%s: Basic %s\r\n",tag,buf64);
+   SendBasicAuth(tag,buf);
 }
 
 void Http::SendAuth()
@@ -367,6 +376,34 @@ void Http::SendAuth()
       SendBasicAuth("Proxy-Authorization",proxy_user,proxy_pass);
    if(user && pass && !(hftp && !QueryBool("use-authorization",proxy)))
       SendBasicAuth("Authorization",user,pass);
+   else if(!hftp)
+      SendBasicAuth("Authorization",Query("authorization",hostname));
+}
+void Http::SendCacheControl()
+{
+   const char *cc_setting=Query("cache-control",hostname);
+   const char *cc_no_cache=(no_cache || no_cache_this)?"no-cache":0;
+   if(!cc_setting && !cc_no_cache)
+      return;
+   int cc_no_cache_len=xstrlen(cc_no_cache);
+   if(cc_no_cache && cc_setting)
+   {
+      const char *pos=strstr(cc_setting,cc_no_cache);
+      if(pos && (pos==cc_setting || pos[-1]==' ')
+	     && (pos[cc_no_cache_len]==0 || pos[cc_no_cache_len]==' '))
+	 cc_no_cache=0, cc_no_cache_len=0;
+   }
+   char *cc=string_alloca(xstrlen(cc_setting)+1+cc_no_cache_len+1);
+   cc[0]=0;
+   if(cc_no_cache)
+      strcpy(cc,cc_no_cache);
+   if(cc_setting)
+   {
+      if(cc[0])
+	 strcat(cc," ");
+      strcat(cc,cc_setting);
+   }
+   Send("Cache-Control: %s\r\n",cc);
 }
 
 bool Http::ModeSupported()
@@ -377,6 +414,7 @@ bool Http::ModeSupported()
    case QUOTE_CMD:
    case RENAME:
    case LIST:
+   case MP_LIST:
    case CHANGE_MODE:
       return false;
    case CONNECT_VERIFY:
@@ -475,6 +513,7 @@ void Http::SendRequest(const char *connection,const char *f)
       }
    case RENAME:
    case LIST:
+   case MP_LIST:
    case CHANGE_MODE:
       abort(); // unsupported
 
@@ -542,10 +581,8 @@ void Http::SendRequest(const char *connection,const char *f)
    }
    SendAuth();
    if(no_cache || no_cache_this)
-   {
       Send("Pragma: no-cache\r\n"); // for HTTP/1.0 compatibility
-      Send("Cache-Control: no-cache\r\n");
-   }
+   SendCacheControl();
    if(mode==ARRAY_INFO && !use_head)
       connection="close";
    else if(mode!=STORE)
@@ -727,6 +764,12 @@ void Http::HandleHeaderLine(const char *name,const char *value)
       SetSuggestedFileName(filename);
       return;
    }
+   if(!strcasecmp(name,"Content-Type"))
+   {
+      xfree(entity_content_type);
+      entity_content_type=xstrdup(value);
+//       const char *cs=strstr(value,"charset=");
+   }
 }
 
 static
@@ -765,7 +808,7 @@ void Http::GetBetterConnection(int level)
       {
 	 if(level<2)
 	    continue;
-	 if(!connection_takeover || o->priority>=priority)
+	 if(!connection_takeover || (o->priority>=priority && !o->suspended))
 	    continue;
 	 o->Disconnect();
 	 return;
@@ -1217,11 +1260,24 @@ int Http::Do()
 
 		  if(status_code/100==5) // server failed, try another
 		     NextPeer();
+		  if(status_code==504) // Gateway Timeout
+		  {
+		     const char *cc=Query("cache-control");
+		     if(cc && strstr(cc,"only-if-cached"))
+		     {
+			if(mode!=ARRAY_INFO)
+			{
+			   SetError(NO_FILE,_("Object is not cached and http:cache-control has only-if-cached"));
+			   return MOVED;
+			}
+			status_code=406;  // so that no retry will be attempted
+		     }
+		  }
 		  // check for retriable codes
-		  if(status_code==408 // Request Timeout
-		  || status_code==502 // Bad Gateway
-		  || status_code==503 // Service Unavailable
-		  || status_code==504)// Gateway Timeout
+		  if(status_code==408  // Request Timeout
+		  || status_code==502  // Bad Gateway
+		  || status_code==503  // Service Unavailable
+		  || status_code==504) // Gateway Timeout
 		  {
 		     Disconnect();
 		     return MOVED;
@@ -1325,7 +1381,7 @@ int Http::Do()
 	       sprintf(err,"%s (%s)",status+status_consumed,file);
 	    else
 	       sprintf(err,"%s (%s%s)",status+status_consumed,cwd,
-		  (cwd[0] && cwd[strlen(cwd)-1]=='/')?"":"/");
+				       (last_char(cwd)=='/')?"":"/");
 	 }
 	 state=RECEIVING_BODY;
 	 SetError(code,err);
@@ -1369,12 +1425,15 @@ int Http::Do()
       }
       else
       {
-	 recv_buf->Resume();
+	 if(recv_buf->IsSuspended())
+	 {
+	    recv_buf->Resume();
+	    if(recv_buf->Size()>0 || (recv_buf->Size()==0 && recv_buf->Eof()))
+	       m=MOVED;
+	 }
 	 BumpEventTime(send_buf->EventTime());
 	 BumpEventTime(recv_buf->EventTime());
-	 if(recv_buf->Size()>0 || (recv_buf->Size()==0 && recv_buf->Eof()))
-	    m=MOVED;
-	 else
+	 if(recv_buf->Size()==0)
 	 {
 	    // check if ranges were emulated by squid
 	    bool no_ranges_if_timeout=(bytes_received==0 && !seen_ranges_bytes);
@@ -1595,7 +1654,7 @@ int Http::Buffered()
 {
    if(mode!=STORE || !send_buf)
       return 0;
-   return send_buf->Size()/*+SocketBuffered(sock)*/;
+   return send_buf->Size()+SocketBuffered(sock);
 }
 
 int Http::Write(const void *buf,int size)
@@ -1620,6 +1679,13 @@ int Http::Write(const void *buf,int size)
    }
    if(size+send_buf->Size()>=max_buf)
       size=max_buf-send_buf->Size();
+   if(entity_size!=NO_SIZE && pos+size>entity_size)
+   {
+      size=entity_size-pos;
+      // tried to write more than originally requested. Make it retry with Open:
+      if(size==0)
+	 return STORE_FAILED;
+   }
    if(size<=0)
       return 0;
 
