@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: NetAccess.cc,v 1.68 2004/05/28 07:57:05 lav Exp $ */
+/* $Id: NetAccess.cc,v 1.70 2004/08/03 11:43:23 lav Exp $ */
 
 #include <config.h>
 
@@ -804,6 +804,8 @@ do_again:
       result->rewind();
       for(file=result->curr(); file!=0; file=result->next())
       {
+	 cur->size=NO_SIZE;
+	 cur->time=NO_DATE;
 	 cur->get_size = need_size && !(file->defined & file->SIZE);
 	 cur->get_time = need_time && (!(file->defined & file->DATE)
 				 || (file->date_prec>0 && can_get_prec_time));
@@ -861,15 +863,12 @@ do_again:
 	 return m;
       if(res==FA::IN_PROGRESS)
 	 return m;
-      assert(res==FA::OK);
       session->Close();
-
-      cur=get_info;
       for(cur=get_info; get_info_cnt-->0; cur++)
       {
-	 if(cur->time!=(time_t)-1)
+	 if(cur->time!=NO_DATE)
 	    result->SetDate(cur->file,cur->time,0);
-	 if(cur->size!=-1)
+	 if(cur->size!=NO_SIZE)
 	    result->SetSize(cur->file,cur->size);
       }
       xfree(get_info);
