@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: SMTask.cc,v 1.28 2004/05/21 09:57:40 lav Exp $ */
+/* $Id: SMTask.cc,v 1.30 2004/09/06 09:19:57 lav Exp $ */
 
 #include <config.h>
 #include <assert.h>
@@ -152,6 +152,12 @@ void SMTask::Schedule()
       SMTask *to_delete=0;
       if(current->deleting)
 	 to_delete=current;
+      if(res!=MOVED && current->block.GetTimeout()==0)
+	 res=MOVED;
+#if 0
+      if(res==MOVED)
+	 printf("MOVED:%p\n",current);
+#endif
       Leave(current);	// unmark it running and change current.
 
       delete to_delete;
@@ -215,6 +221,11 @@ int SMTask::TaskCount()
    return count;
 }
 
+void SMTask::Cleanup()
+{
+  delete init_task;
+}
+
 #include <errno.h>
 #include "ResMgr.h"
 ResDecl enospc_fatal ("xfer:disk-full-fatal","no",ResMgr::BoolValidate,ResMgr::NoClosure);
