@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: commands.cc,v 1.211 2004/06/02 06:16:29 lav Exp $ */
+/* $Id: commands.cc,v 1.213 2004/08/03 11:34:39 lav Exp $ */
 
 #include <config.h>
 
@@ -2604,8 +2604,7 @@ CMD(find)
 
    if(!args->getcurr())
       args->Append(".");
-   FinderJob_List *j=new class FinderJob_List(session->Clone(),args,
-      output?output:new FDStream(1,"<stdout>"));
+   FinderJob_List *j=new class FinderJob_List(session->Clone(),args,output);
    j->set_maxdepth(maxdepth);
    args=0;
    output=0;
@@ -2741,8 +2740,7 @@ CMD(du)
 
    if(!args->getcurr())
       args->Append(".");
-   FinderJob_Du *j=new class FinderJob_Du(session->Clone(),args,
-      output?output:new FDStream(1,"<stdout>"));
+   FinderJob_Du *j=new class FinderJob_Du(session->Clone(),args,output);
    args=0;
    j->PrintDepth(maxdepth);
    j->SetBlockSize(blocksize);
