@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: resource.cc,v 1.104 2004/05/26 12:31:43 lav Exp $ */
+/* $Id: resource.cc,v 1.108 2004/08/23 14:39:37 lav Exp $ */
 
 #include <config.h>
 
@@ -187,6 +187,7 @@ static ResDecl
    ResDecl06 ("ftp:port-range",		  "full",  ResMgr::RangeValidate,0),
    ResDecl06a("ftp:port-ipv4",		  "",	   ResMgr::IPv4AddrValidate,0),
    ResDecl07 ("ftp:proxy",		  "",	   FtpProxyValidate,0),
+   ResDecl07a("ftp:proxy-auth-joined",	  "no",	   ResMgr::BoolValidate,0),
    ResDecl08 ("ftp:rest-list",		  "no",	   ResMgr::BoolValidate,0),
    ResDecl09 ("ftp:rest-stor",		  "yes",   ResMgr::BoolValidate,0),
    ResDecl09a("ftp:timezone",		  "GMT",   0,0),
@@ -227,6 +228,7 @@ static ResDecl
    ResDecl20a("ftp:retry-530",		  RETRY_530,ResMgr::ERegExpValidate,0),
    ResDecl20b("ftp:retry-530-anonymous",  "Login incorrect",ResMgr::ERegExpValidate,0),
    ResDecl21 ("hftp:cache",		  "yes",   ResMgr::BoolValidate,0),
+   ResDecl21a("hftp:cache-control",	  "",	   0,0),
    ResDecl22 ("hftp:proxy",		  "",	   HttpProxyValidate,0),
    ResDecl22a("hftp:use-authorization",	  "yes",   ResMgr::BoolValidate,0),
    ResDecl23 ("hftp:use-head",		  "yes",   ResMgr::BoolValidate,0),
@@ -234,7 +236,9 @@ static ResDecl
    ResDecl24a("http:accept",		  "*/*",   0,0),
    ResDecl24b("http:accept-language",	  "",	   0,0),
    ResDecl24c("http:accept-charset",	  "",	   0,0),
+   ResDecl24e("http:authorization",	  "",	   0,0),
    ResDecl24 ("http:cache",		  "yes",   ResMgr::BoolValidate,0),
+   ResDecl24d("http:cache-control",	  "",	   0,0),
    ResDecl25 ("http:proxy",		  "",	   HttpProxyValidate,0),
    ResDecl26 ("http:user-agent",	  PACKAGE"/"VERSION,0,0),
    ResDecl27 ("http:cookie",		  "",	   0,0),
@@ -273,7 +277,7 @@ static ResDecl
    ResDecl50 ("mirror:use-pget-n",	  "1",	   ResMgr::UNumberValidate,ResMgr::NoClosure);
 
 static ResDecl
-   res_sftp_packets  ("sftp:max-packets-in-flight","8",ResMgr::UNumberValidate,0),
+   res_sftp_packets  ("sftp:max-packets-in-flight","16",ResMgr::UNumberValidate,0),
    res_sftp_proto_ver("sftp:protocol-version","4",ResMgr::UNumberValidate,0),
    res_sftp_rsize    ("sftp:size-read", "0x8000",ResMgr::UNumberValidate,0),
    res_sftp_wsize    ("sftp:size-write","0x8000",ResMgr::UNumberValidate,0),
